@@ -33,7 +33,7 @@ strcuts(char *dst, const char *src, const char start, const char end,
 
         for (i = 0, j = 0; src[i] != start && start != '\0'; i++) {
 		/* start not found */
-		if (i > len)
+		if (i >= len)
 			return -1;
 	}
 
