@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, 2005, 2006 Marcus Glocker <marcus@nazgul.ch>
+ * Copyright (c) 2004 - 2009 Marcus Glocker <marcus@nazgul.ch>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,6 +14,7 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include <stdint.h>
 #include <signal.h>
 
 #include "config.h"
@@ -73,7 +74,8 @@ extern const char *http_fn_ran;
 extern const char *http_fn_rac;
 extern const char *http_fn_aen;
 extern const char *http_fv_srv;
-extern const char *http_fv_pro;
+extern const char *http_fv_pr0;
+extern const char *http_fv_pr1;
 extern const char *http_fv_cgi;
 extern const char *http_fv_teg;
 extern const char *http_fv_lch;
@@ -160,8 +162,8 @@ struct header {
 	char	rp_fv_loc[1024];	/* Response Value Location: */
 	char	rp_fv_auw[1024];	/* Response Value WWW-Authenticate: */
 	char	rp_header[8192];	/* Response Header */
-	off_t	rp_fsize;		/* Repsonse File Size */
-	off_t	rp_foffs;		/* Response File Offset */
+	intmax_t	rp_fsize;	/* Repsonse File Size */
+	intmax_t	rp_foffs;	/* Response File Offset */
 	int	rp_hsize;		/* Response Header Size */
 	us_int	rp_status;		/* Response Status */
 	us_int	x_chk;			/* Flag for Chunking */
@@ -193,13 +195,13 @@ struct connection {
 	char	ip[24];			/* Connection, Client IP */
 	char	*rbuf;			/* Connection, Read buf */
 	char	*wbuf;			/* Connection, Write buf */
-	off_t	roff;			/* Connection, Read buf offset */
-	off_t	wred;			/* Connection, Write buf read bytes */
-	off_t	wsnd;			/* Connection, Write buf sent bytes */
+	intmax_t	roff;		/* Connection, Read buf offset */
+	intmax_t	wred;		/* Connection, Write buf read bytes */
+	intmax_t	wsnd;		/* Connection, Write buf sent bytes */
 	int	to;			/* Connection, Timeout */
 	int	pfdo;			/* Connection, Open File descriptors */
 	int	pfdn[HDN];		/* Partitial, File descriptor */
-	off_t	pfds[HDN];		/* Partitial, Bytes sent */
+	intmax_t	pfds[HDN];	/* Partitial, Bytes sent */
 	int	psta[HDN];		/* Partitial, Response Status */
 	char	*pfdh[HDN];		/* Partitial, Response Header */
 	char	*plreq[HDN];		/* Partitial, Log Request */
