@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, 2005, 2006 Marcus Glocker <marcus@nazgul.ch>
+ * Copyright (c) 2004 - 2009 Marcus Glocker <marcus@nazgul.ch>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -98,12 +98,12 @@ static const char *doc =
 int
 http_verify(const char *header, const char *cip, const int sfd, const int hr)
 {
-	int		r;
-	char		*h, *b, line[1024];
+	int		r, proto;
+	char		*h, *b, line[1024], protocol[16];
 	time_t		tnow;
 	struct tm	*t;
 
-	r = 0;
+	r = proto = 0;
 
 	/* check for valid method */
 	if (strcutl(line, header, 1, sizeof(line)) > 0) {
@@ -142,6 +142,45 @@ http_verify(const char *header, const char *cip, const int sfd, const int hr)
 		free(b);
 		return (0);
 	}
+
+	/* check for valid protocol version */
+	r = 0;
+	strcutw(protocol, line, 3, sizeof(protocol));
+	if (!strcasecmp(protocol, http_fv_pr0)) {
+		r = 1;
+		proto = 0;
+	} else if (!strcasecmp(protocol, http_fv_pr1)) {
+		r = 1;
+		proto = 1;
+	}
+	if (r == 0) {
+		h = http_head(http_s_501, line, cip, 0);
+		b = http_body(http_s_501, "", h, 0);
+		c[sfd].pfdo++;
+		c[sfd].pfdn[hr] = 1;
+		c[sfd].pfdh[hr] = strdup(b);
+		c[sfd].x_ful[hr] = 1;
+		c[sfd].x_chk[hr] = 0;
+		c[sfd].x_sta = 0;
+		free(h);
+		free(b);
+		return (r);
+	}
+
+	/* check if host header option exists for HTTP/1.1 */
+	if (proto == 1 && http_headeropt_exist(header, "Host:") != 1) {
+		h = http_head(http_s_400, line, cip, 0);
+		b = http_body(http_s_400, "", h, 0);
+		c[sfd].pfdo++;
+		c[sfd].pfdn[hr] = 1;
+		c[sfd].pfdh[hr] = strdup(b);
+		c[sfd].x_ful[hr] = 1;
+		c[sfd].x_chk[hr] = 0;
+		c[sfd].x_sta = 0;
+		free(h);
+		free(b);
+		return (0);
+	}
 	
 	/* access log 1 client ip, date, request string */
 	time(&tnow);
@@ -193,7 +232,7 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 	if (rh->rp_status == 401) {
 		x = http_body(http_s_401, "", rh->rp_header, rh->x_chk);
 
-		if (config.c401[0] != '0') {
+		if (config.c401[0] != '\0') {
 			if (rh->x_dov)
 				snprintf(cpage, sizeof(cpage), "%s/%s",
 				    rh->rq_docrootv, config.c401);
@@ -206,7 +245,7 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 	if (rh->rp_status == 403) {
 		x = http_body(http_s_403, "", rh->rp_header, rh->x_chk);
 
-		if (config.c403[0] != '0') {
+		if (config.c403[0] != '\0') {
 			if (rh->x_dov)
 				snprintf(cpage, sizeof(cpage), "%s/%s",
 				    rh->rq_docrootv, config.c403);
@@ -219,7 +258,7 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 	if (rh->rp_status == 404) {
 		x = http_body(http_s_404, "", rh->rp_header, rh->x_chk);
 
-		if (config.c404[0] != '0') {
+		if (config.c404[0] != '\0') {
 			if (rh->x_dov)
 				snprintf(cpage, sizeof(cpage), "%s/%s",
 				    rh->rq_docrootv, config.c404);
@@ -394,14 +433,18 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 			    rh->rq_script);
 			cgienv[i++] = sys_benv("SERVER_ADMIN=%s",
 			    config.serveradmin);
-			cgienv[i++] = sys_benv("SERVER_NAME=%s",
-			    config.servername);
 			cgienv[i++] = sys_benv("SERVER_PROTOCOL=%s",
-			    http_fv_pro);
+			    http_fv_pr1);
 			cgienv[i++] = sys_benv("SERVER_SIGNATURE=%s",
 			    http_sig);
 			cgienv[i++] = sys_benv("SERVER_SOFTWARE=%s",
 			    http_fv_srv);
+			if (rh->x_hos)
+				cgienv[i++] = sys_benv("SERVER_NAME=%s",
+				    rh->rq_fv_hos);
+			else
+				cgienv[i++] = sys_benv("SERVER_NAME=%s",
+				    config.servername);
 			if (rh->x_cte)
 				cgienv[i++] = sys_benv("CONTENT_TYPE=%s",
 				    rh->rq_fv_cte);
@@ -725,9 +768,9 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 			/* create html file entry */
 			snprintf(tmp, sizeof(tmp),
 			    "<tr><td>%s</td><td><a href=\"%s\">%s</a>"
-			    "</td><td>%s</td><td>%lld</td></tr>\n",
+			    "</td><td>%s</td><td>%jd</td></tr>\n",
 			    image, dirsort[i], dirsort[i], http_date(t),
-			    sta.st_size);
+			    (intmax_t)sta.st_size);
 
 			/* buffer full? send it! */
 			if (strlen(tmp) > sizeof(buf) - strlen(buf)) {
@@ -1011,7 +1054,7 @@ http_access_htaccess(char *dst, const char *rootdir, const int dsize)
 	struct stat	s;
 
 	/* config: htaccess */
-	if (config.htaccess[0] == '0')
+	if (config.htaccess[0] == '\0')
 		return (0);
 
 	memset(path, 0, sizeof(path));
@@ -1051,7 +1094,7 @@ http_alog(const int sfd, const int hr)
 	time(&tnow);
 	t = localtime(&tnow);
 
-	snprintf(log_1, sizeof(log_1), "%s - - %s \"%s\" %d %lld",
+	snprintf(log_1, sizeof(log_1), "%s - - %s \"%s\" %d %jd",
 	    c[sfd].ip, sys_date(t), c[sfd].plreq[hr], c[sfd].psta[hr],
 	    c[sfd].pfds[hr]);
 	snprintf(log_2, sizeof(log_2), "\"%s\" \"%s\"",
@@ -1062,6 +1105,27 @@ http_alog(const int sfd, const int hr)
 	return (0);
 }
 
+/*
+ * http_headeropt_exist()
+ *	check if a specific header option exists
+ * Return:
+ *	0 = header option doesn't exist, 1 = header option exists
+ */
+int
+http_headeropt_exist(const char *header_data, char *opt)
+{
+	int	i;
+	char	line[1024], option[1024];
+
+	for (i = 2; strcutl(line, header_data, i, sizeof(line)) != -1; i++) {
+		strcutw(option, line, 1, sizeof(option));
+		if (!strcasecmp(option, opt))
+			return (1);
+	}
+
+	return (0);
+}
+
 /*
  * http_chunk()
  *	add chunk information to a data block
@@ -1192,7 +1256,7 @@ http_head(const char *status, const char *request, const char *cip,
 
 	/* status Date: Server: */
 	snprintf(header, sizeof(header), "%s %s\r\n%s %s\r\n%s %s\r\n",
-	    http_fv_pro, status, http_fn_dat, date, http_fn_srv, http_fv_srv);
+	    http_fv_pr1, status, http_fn_dat, date, http_fn_srv, http_fv_srv);
 	/* Connection: */
 	snprintf(tmp, sizeof(tmp), "%s ", http_fn_con);
 	strlcat(header, tmp, sizeof(header));
@@ -1296,7 +1360,7 @@ struct header *
 http_header(const char *header_data, const char *force_status,
     const int force_status_nr, const int blen, const int sfd)
 {
-	off_t		fsize;
+	intmax_t	fsize;
 	int		file, i, r;
 	char		line[1024], option[1024];
 	char		alias[1024], cgi_full[1024];
@@ -1347,7 +1411,7 @@ http_header(const char *header_data, const char *force_status,
 	strcutw(h->rq_protocol, line, 3, sizeof(h->rq_protocol));
 
 	/* set protocol depended flags */
-	if (!strcasecmp(h->rq_protocol, http_fv_pro))
+	if (!strcasecmp(h->rq_protocol, http_fv_pr1))
 		h->x_chk = 1;
 
 	/* decode uri if encoded */
@@ -1456,21 +1520,23 @@ http_header(const char *header_data, const char *force_status,
 	}
 
 	/* cut possible alias from uri */
-	strcuts(alias, h->rq_uri, '\0', '/', sizeof(alias));
+	if (strcuts(alias, h->rq_uri, '\0', '/', sizeof(alias)) == -1)
+		strlcpy(alias, h->rq_uri, sizeof(alias));
 
 	/* check if uri contains an alias */
-	if (fparse(docroot, alias, config.file, sizeof(docroot)) != -1)
+	if (fparse(docroot, alias, config.file, sizeof(docroot)) != -1) {
 		/* yes cut it out */
-		strcuti(file_path, h->rq_uri, strlen(alias),
-		    strlen(h->rq_uri), sizeof(file_path));
-	else {
+		if (strcuti(file_path, h->rq_uri, strlen(alias),
+		    strlen(h->rq_uri), sizeof(file_path)) == -1)
+			bzero(file_path, sizeof(file_path));
+	} else {
 		/* no keep defaults */
 		strlcpy(docroot, h->rq_docrootv, sizeof(docroot));
 		strlcpy(file_path, h->rq_uri, sizeof(file_path));
 	}
 
 	/* homedirs */
-	if (config.homedirs[0] != '0') {
+	if (config.homedirs[0] != '\0') {
 		if (h->rq_uri[1] == '~') {
 			strlcpy(docroot, config.homedirs, sizeof(docroot));
 			strlcpy(file_path, h->rq_uri, sizeof(file_path));
@@ -1706,7 +1772,7 @@ http_header(const char *header_data, const char *force_status,
 
 	/* Status, Date:, Server: */
 	snprintf(h->rp_header, sizeof(h->rp_header),
-	    "%s %s\r\n%s %s\r\n%s %s\r\n", http_fv_pro, status, http_fn_dat,
+	    "%s %s\r\n%s %s\r\n%s %s\r\n", http_fv_pr1, status, http_fn_dat,
 	    h->rp_fv_dat, http_fn_srv, http_fv_srv);
 	/* Connection: */
 	snprintf(tmp, sizeof(tmp), "%s ", http_fn_con);
@@ -1731,13 +1797,13 @@ http_header(const char *header_data, const char *force_status,
 	/* Last-Modified:, Content-Length: */
 	if ((h->rp_status == 200 || h->rp_status == 206) && h->x_cgi == 0 &&
 	    h->x_idx == 0) {
-		snprintf(tmp, sizeof(tmp), "%s %s\r\n%s %lld\r\n",
+		snprintf(tmp, sizeof(tmp), "%s %s\r\n%s %jd\r\n",
 		    http_fn_lmd, h->rp_fv_dam, http_fn_clt, h->rp_fsize);
 		strlcat(h->rp_header, tmp, sizeof(h->rp_header));
 	}
 	/* Content-Range: */
 	if (h->rp_status == 206 && h->x_cgi == 0 && h->x_idx == 0) {
-		snprintf(tmp, sizeof(tmp), "%s bytes %lld-%lld/%lld\r\n",
+		snprintf(tmp, sizeof(tmp), "%s bytes %jd-%jd/%jd\r\n",
 		    http_fn_rac, h->rp_foffs, fsize - 1, fsize);
 		strlcat(h->rp_header, tmp, sizeof(h->rp_header));
 	}
@@ -1763,8 +1829,10 @@ http_header(const char *header_data, const char *force_status,
 		strlcat(h->rp_header, tmp, sizeof(h->rp_header));
 	}
 	/* end of header */
-	if (h->x_cgi == 0 || h->rp_status == 403 || h->rp_status == 404)
-		strlcat(h->rp_header, "\r\n", sizeof(h->rp_header));
+	if (h->x_cgi == 0 || h->rp_status == 403 || h->rp_status == 404) {
+		if (force_status_nr == 0)
+			strlcat(h->rp_header, "\r\n", sizeof(h->rp_header));
+	}
 
 	/* return the size of our header */
 	h->rp_hsize = strlen(h->rp_header);
