@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, 2005, 2006 Marcus Glocker <marcus@nazgul.ch>
+ * Copyright (c) 2004 - 2009 Marcus Glocker <marcus@nazgul.ch>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -109,8 +109,9 @@ const char *http_fn_auw		= "WWW-Authenticate:";
 const char *http_fn_ran		= "Range:";
 const char *http_fn_rac		= "Content-Range:";
 const char *http_fn_aen		= "Accept-Encoding:";
-const char *http_fv_srv		= "nostromo 1.9";
-const char *http_fv_pro		= "HTTP/1.1";
+const char *http_fv_srv		= "nostromo 1.9.1";
+const char *http_fv_pr0	=	 "HTTP/1.0";
+const char *http_fv_pr1	=	 "HTTP/1.1";
 const char *http_fv_cgi		= "CGI/1.1";
 const char *http_fv_teg		= "chunked";
 const char *http_fv_lch		= "0\r\n\r\n";
@@ -531,13 +532,13 @@ main(int argc, char *argv[])
 	}
 	if (fparse(config.user, "user", config.file, sizeof(config.user))
 	    == -1)
-		strlcpy(config.user, "0", sizeof(config.user));
+		config.user[0] = '\0';
 	if (fparse(config.htaccess, "htaccess", config.file,
 	    sizeof(config.htaccess)) == -1)
-		strlcpy(config.htaccess, "0", sizeof(config.htaccess));
+		config.htaccess[0] = '\0';
 	if (fparse(config.htpasswd, "htpasswd", config.file,
 	    sizeof(config.htpasswd)) == -1)
-		strlcpy(config.htpasswd, "0", sizeof(config.htpasswd));
+		config.htpasswd[0] = '\0';
 #ifdef __OpenBSD__
 	else {
 		if (strcmp(config.htpasswd, "+bsdauth") == 0)
@@ -568,16 +569,16 @@ main(int argc, char *argv[])
 	}
 	if (fparse(config.c401, "custom_401", config.file, sizeof(config.user))
 	    == -1)
-		strlcpy(config.c401, "0", sizeof(config.c401));
+		config.c401[0] = '\0';
 	if (fparse(config.c403, "custom_403", config.file, sizeof(config.user))
 	    == -1)
-		strlcpy(config.c403, "0", sizeof(config.c403));
+		config.c403[0] = '\0';
 	if (fparse(config.c404, "custom_404", config.file, sizeof(config.user))
 	    == -1)
-		strlcpy(config.c404, "0", sizeof(config.c404));
+		config.c404[0] = '\0';
 	if (fparse(config.homedirs, "homedirs", config.file,
 	    sizeof(config.homedirs)) == -1)
-		strlcpy(config.homedirs, "0", sizeof(config.homedirs));
+		config.homedirs[0] = '\0';
 
 	/* we need all mandatory parameters otherwise no start */
 	if (r == -1)
@@ -592,7 +593,7 @@ main(int argc, char *argv[])
 	/*
 	 * get UID and GID
 	 */
-	if (config.user[0] != '0') {
+	if (config.user[0] != '\0') {
 		if ((pwd = getpwnam(config.user)) == NULL)
 			errx(1, "%s: no such user.", config.user);
 	}
@@ -911,7 +912,7 @@ main(int argc, char *argv[])
 			sys_close_except(fdbsdauth[1]);
 			grp = getgrnam("auth");
 			setgid(grp->gr_gid);
-			if (config.user[0] != '0')
+			if (config.user[0] != '\0')
 				setuid(pwd->pw_uid);
 			sys_daemon_bsdauth();
 			exit(0);
@@ -935,7 +936,7 @@ main(int argc, char *argv[])
 	/*
 	 * set UID and GID
 	 */
-	if (config.user[0] != '0') {
+	if (config.user[0] != '\0') {
 		if (setgid(pwd->pw_gid) == -1) {
 			syslog(LOG_ERR, "setgid: %s", strerror(errno));
 			exit(1);
