@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, 2005, 2006 Marcus Glocker <marcus@nazgul.ch>
+ * Copyright (c) 2004 - 2009 Marcus Glocker <marcus@nazgul.ch>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -580,10 +580,14 @@ sys_close_except(const int fd)
 void
 sys_log(const int debug, const char *string, ...)
 {
+	va_list ap;
+
 	if (!debug)
 		return;
 
-	syslog(LOG_DEBUG, "%s", string);
+	va_start(ap, string);
+	vsyslog(LOG_INFO, string, ap);
+	va_end(ap);
 }
 
 /*
